-- Wallet is a simple money and/or destiny point display

-- Currency - store all of the info for each currency, Currency[1] and Currency[2] are for "Gold" and "DP"
-- -- format {Name, Description, Image, ImageWidth, ImageHeight, MaxQuantity, IsAccountItem, DefaultOrder} (default order will be the order of the last loaded character - this will make using the same order on more than one character easier)
-- -- as new currencies are added, they get ID=#Currency+1, DefaultOrder=#Currency+1
-- -- on load we can sort them by ID so that the Index will equal the ID

-- walletSettings - store the per character display info
-- -- format {CurrencyIndex (saved but recreated each load), Show, Hide0, Order, Dock, Left, Top}

-- to do:
--add event handler to Wallet to catch new currencies
-- -- need to test


import "GaranStuff.Wallet.Class";
import "GaranStuff.Wallet.Table";
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "GaranStuff.Wallet.VindarPatch";
import "GaranStuff.Wallet.RadioButtonGroup";

euroFormat=(tonumber("1,000")==1);

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

walletOpacity=.9;

walletHUDState=true;
walletState={};
walletMoveToggle=false;
localPlayer=Turbine.Gameplay.LocalPlayer:GetInstance();
playerWallet=localPlayer:GetWallet();
goldWalletIndex=1;
walletSize=playerWallet:GetSize(); -- used to track initialization
walletSettings={};

function AddCurrency(currency)
	-- attempt to add a new currency
	local currencyName=currency:GetName()
	local tmpIndex
	local found=false;
	for tmpIndex=1,#Currency do
		if Currency[tmpIndex][1]==currencyName then
			-- update the Description and SmallImage values incase Turbine changes them
			Currency[tmpIndex][2]=currency:GetDescription();
			local Image=currency:GetSmallImage();
			if Image~=nil then
				Currency[tmpIndex][3]=Image;
				Currency[tmpIndex][4]=16;
				Currency[tmpIndex][5]=16;
			end
			found=true;
			break;
		end
	end
	if not found then
		-- add this currency to the end of the Currency table
		local smallImage=currency:GetSmallImage();
		local x,y
		if smallImage==nil then
			x=0;
			y=0;
		else
			x=16;
			y=16;
		end
		Currency[#Currency+1]={currencyName, currency:GetDescription(), smallImage, x, y, currency:GetMaxQuantity(), currency:IsAccountItem(),#Currency+1};
	end
	return (not found)
end
function LoadCurrency()
	-- load all previously known currencies
	Currency=PatchDataLoad( Turbine.DataScope.Account, "Wallet_Currency");
	if Currency==nil then
		-- create the default currency entries for gold, DP, Commendation, Shard, Mark, Medallion, Seal so that they filter to the top by default
		-- format {ID, Name, Description, SmallImage, ImageWidth, ImageHeight, MaxQuantity, IsAccountItem, DefaultOrder} (default order will be the order of the last loaded character - this will make using the same order on more than one character easier)
		Currency={{"Gold", "This is your character's gold, silver and copper coins.  Coinage can be spent at many vendors.", 0x41005e9e, 16, 16, 0, false, 1},{"DP", "These are the Destiny Points you have earned. DP can be spent on Perks", 0x4100a683, 15, 14, 0, true, 2},{"Commendation", "Points awarded for victories against opposing forces in Monster Player battles.", 1091712150, 16, 16, 10000, true, 3},{"Shard", "A shard of a relic. This can be used at Relic-masters.", 1091695426, 16, 16, 9999999, true, 4},{"Mark", "Marks representing your successes. These can be traded at Skirmish camps.", 1091694709, 16, 16, 9999999, true, 5},{"Medallion", "Medallions representing your triumphs. These can be bartered at Skirmish camps and at class trainers.", 1091694710, 16, 16, 9999999, true, 6},{"Seal", "Seals representing your victories. These can be bartered at Skirmish camps and at class trainers.", 1091694711, 16, 16, 9999999, true, 7}};
	end
	-- now iterate through the current player's wallet adding any previously unidentified currencies
	local tmpCurrency;
	for tmpCurrency=1,playerWallet:GetSize() do
		AddCurrency(playerWallet:GetItem(tmpCurrency));
	end
end
LoadCurrency()

LoadSettings=function()
-- walletSettings - store the per character display info
-- -- format {CurrencyID, Show, Hide0, Order, Dock, Left, Top, windowHandle (not saved)}

	local settings=PatchDataLoad( Turbine.DataScope.Character, "Wallet_Settings");
	local tmpCurrency;
	if settings~=nil then
		local tmpIndex;
		for tmpIndex=1,#settings do
			if settings[tmpIndex][1]=="Opacity" then
				if euroFormat then
					-- replace "." with "," tonumber(string.gsub(tmpSettings[index][2].R,".",","))
					walletOpacity=tonumber((string.gsub(settings[tmpIndex][2],"%.",",")));
				else
					walletOpacity=tonumber((string.gsub(settings[tmpIndex][2],",",".")));
				end
			elseif type(settings[tmpIndex][2])=="table" then
				local tmpName;
				for tmpCurrency=1, #Currency do
					tmpName=Currency[tmpCurrency][1];
					local newIndex=#walletSettings+1;
					if settings[tmpIndex][1]==tmpName then
						walletSettings[newIndex]={};
						walletSettings[newIndex][1]=tmpCurrency;
						walletSettings[newIndex][2]=settings[tmpIndex][2][2];
						walletSettings[newIndex][3]=settings[tmpIndex][2][3];
						walletSettings[newIndex][4]=settings[tmpIndex][2][4];
						walletSettings[newIndex][5]=settings[tmpIndex][2][5];
						walletSettings[newIndex][6]=settings[tmpIndex][2][6];
						walletSettings[newIndex][7]=settings[tmpIndex][2][7];
					end
				end
			end
		end
		-- now create walletSettings for any Currencies that were not previously saved for this character
		for tmpCurrency=1,#Currency do
			local found=false;
			for tmpIndex=1,#walletSettings do
				if walletSettings[tmpIndex][1]==tmpCurrency then
					found=true;
					break;
				end
			end
			if not found then
				local newIndex=#walletSettings+1;
				walletSettings[newIndex]={tmpCurrency, (tmpCurrency<=7), true, newIndex, true, 0, tmpCurrency*20};
			end
		end
	else
		-- create walletSettings for all Currencies - use the DefaultOrder since there is no chance of collisions
		for tmpCurrency=1,#Currency do
			local newIndex=#walletSettings+1;
			walletSettings[newIndex]={tmpCurrency, (tmpCurrency<=7), true, Currency[tmpCurrency][8], (newIndex>1), 0, tmpCurrency*20};
		end
	end
	-- now sort the wallet entries by Order
	table.sort(walletSettings,function(arg1,arg2) if arg1[4]<arg2[4] then return(true) end end );
	-- and finally, assign Order to a guaranteed new "corrrect" value - this will allow Order to fix itself if it ever gets out of wack due to a crash
	for tmpCurrency=1,#walletSettings do
		walletSettings[tmpCurrency][4]=tmpCurrency;
	end
end
LoadSettings()

Wallet=class(Turbine.UI.Window);

function Wallet:Constructor(index)
	Turbine.UI.Window.Constructor( self );
	local currencyName=string.lower(Currency[walletSettings[index][1]][1]);
	self.Index=index;
	self.CurrencyIndex=walletSettings[index][1];
	self.Currency=nil;

	if Currency[self.CurrencyIndex][1]=="Gold" then
		goldWalletIndex=index;
	end
	if Currency[self.CurrencyIndex][1]~="Gold" and Currency[self.CurrencyIndex][1]~="DP" then
		local tmpIndex;
		for tmpIndex=1,playerWallet:GetSize() do
			if playerWallet:GetItem(tmpIndex):GetName()==Currency[self.CurrencyIndex][1] then
				self.Currency=playerWallet:GetItem(tmpIndex);
				break;
			end
		end
	end
	self:SetSize(173,60);
--Turbine.Shell.WriteLine("Index:"..tostring(index).." Left:"..tostring(walletSettings[index][6]))
	self:SetPosition(walletSettings[index][6]-1,walletSettings[index][7]); -- we start the window one pixel to the left to force the initial repositioning after all windows are loaded
	self:SetBackColor(Turbine.UI.Color(0,0,0,0)); -- a necessary evil since RoI for some reason. if we don't include this, the window doesn't redraw properly
	self:SetOpacity(walletOpacity);

	self.MovePanel=Turbine.UI.Control();
	self.MovePanel:SetParent(self);
	self.MovePanel:SetSize(173,19);
	self.MovePanel:SetBackColor(Turbine.UI.Color(.5,0,0,0));
	self.Move2=Turbine.UI.Control();
	self.Move2:SetParent(self.MovePanel);
	self.Move2:SetSize(156,12);
	self.Move2:SetPosition(17,2);
	self.Move2:SetBackground("GaranStuff/Wallet/Resources/move2.tga");
	self.Move2:SetMouseVisible(false);
	self.Move=Turbine.UI.Control();
	self.Move:SetParent(self.MovePanel);
	self.Move:SetSize(19,19);
	self.Move:SetBackground("GaranStuff/Wallet/Resources/move.tga");
	self.Move:SetMouseVisible(false);
	self.MoveText=Turbine.UI.Label();
	self.MoveText:SetParent(self.MovePanel);
	self.MoveText:SetSize(156,12);
	self.MoveText:SetPosition(17,2);
	self.MoveText:SetFont(Turbine.UI.Lotro.Font.Verdana10);
	self.MoveText:SetText("Wallet "..Currency[self.CurrencyIndex][1]);
	self.MoveText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	self.MoveText:SetMouseVisible(false);
	self.MovePanel:SetVisible(false);

	self.MoveX=-1;
	self.MoveY=0;
	self.Moving=false;

	self.MovePanel.MouseDown=function()
	end

	self.MovePanel.MouseClick = function(sender,args)
		if self.Moving then
			self.Moving=false;
			self.MoveX=-1;
		end
	end
	self.MovePanel.MouseDown=function(sender,args)
		self.MoveX=args.X;
		self.MoveY=args.Y;
	end
	self.MovePanel.MouseUp=function(sender,args)
		self.MoveX=-1;
		self.MoveY=0;
	end

	self.MovePanel.MouseEnter=function(sender,args)
		self:SetOpacity(1);
	end

	self.MovePanel.MouseLeave=function(sender,args)
		self:SetOpacity(walletOpacity);
	end
	self.MovePanel.MouseMove=function(sender,args)
		if self.MoveX~=-1 and (args.X~=self.MoveX or args.Y~= self.MoveY) then
			self.Moving=true;
			local newLeft=self:GetLeft()-(self.MoveX-args.X)
			local newTop=self:GetTop()-(self.MoveY-args.Y)
			if newLeft<0 then newLeft=0 end;
			if newLeft>(Turbine.UI.Display.GetWidth()-self:GetWidth()) then newLeft=Turbine.UI.Display.GetWidth()-self:GetWidth() end;
			if newTop<-19 then newTop=-19 end;
			if newTop>(Turbine.UI.Display.GetHeight()-self:GetHeight()) then newTop=Turbine.UI.Display.GetHeight()-self:GetHeight() end;

			walletSettings[self.Index][6]=newLeft;
			walletSettings[self.Index][7]=newTop;
			self:SetPosition(newLeft,newTop);
		end
	end

	self.WalletPanel=Turbine.UI.Control();
	self.WalletPanel:SetParent(self);
	self.WalletPanel:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.WalletPanel:SetPosition(0,20);
	self.WalletPanel:SetSize(self:GetWidth(),20);
	self.WalletPanel:SetMouseVisible(false);

	self.Displayed=true;
	if walletSettings[self.Index][2] then
		if walletSettings[self.Index][3] then
			local curQty=0;
			local tmpIndex;
			if Currency[self.CurrencyIndex][1]=="Gold" then
				if localPlayer:GetAttributes().GetMoney~=nil then
					curQty=localPlayer:GetAttributes():GetMoney();
				end
			elseif Currency[self.CurrencyIndex][1]=="DP" then
				if localPlayer:GetAttributes().GetDestinyPoints~=nil then
					curQty=localPlayer:GetAttributes():GetDestinyPoints();
				end
			else
				if self.Currency~=nil then
					curQty=self.Currency:GetQuantity();
				end
			end
			self.Displayed=(curQty~=0);
		end
	else
		self.Displayed=false;
	end
	self:SetVisible(true);

	if self.Displayed then
		self.WalletPanel:SetHeight(20);
	else
		self.WalletPanel:SetHeight(0);
	end

	if Currency[self.CurrencyIndex][1]=="Gold" then
		self.GPText=Turbine.UI.Label();
		self.GPText:SetParent(self.WalletPanel);
		self.GPText:SetSize(30,20);
		self.GPText:SetPosition(0,0);
		self.GPText:SetMultiline(false);
		self.GPText:SetFont(Turbine.UI.Lotro.Font.Verdana16);
		self.GPText:SetText("");
		self.GPText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
		self.GPText:SetMouseVisible(false);
		self.GPText.HScrollBar=Turbine.UI.Lotro.ScrollBar();
		self.GPText.HScrollBar:SetOrientation(Turbine.UI.Orientation.Horizontal);
		self.GPText.HScrollBar:SetParent(self); -- see if the scrollbar will trigger "IsVisible()" even if it has no parent
		self.GPText.HScrollBar:SetHeight(12);
		self.GPText.HScrollBar:SetPosition(0,-13); -- position it off canvas so it won't ever actually display
		self.GPText:SetHorizontalScrollBar(self.GPText.HScrollBar);

		self.GPIcon=Turbine.UI.Control();
		self.GPIcon:SetParent(self.WalletPanel);
		self.GPIcon:SetSize(27,21);
		self.GPIcon:SetPosition(30,0);
		self.GPIcon:SetBackground(0x41007e7b);
		self.GPIcon:SetMouseVisible(false);

		self.SPText=Turbine.UI.Label();
		self.SPText:SetParent(self.WalletPanel);
		self.SPText:SetSize(30,20);
		self.SPText:SetPosition(62,0);
		self.SPText:SetFont(Turbine.UI.Lotro.Font.Verdana16);
		self.SPText:SetText("");
		self.SPText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
		self.SPText:SetMouseVisible(false);
		self.SPIcon=Turbine.UI.Control();
		self.SPIcon:SetParent(self.WalletPanel);
		self.SPIcon:SetSize(27,21);
		self.SPIcon:SetPosition(92,0);
		self.SPIcon:SetBackground(0x41007e7c);
		self.SPIcon:SetMouseVisible(false);

		self.CPText=Turbine.UI.Label();
		self.CPText:SetParent(self.WalletPanel);
		self.CPText:SetSize(22,20);
		self.CPText:SetPosition(124,0);
		self.CPText:SetFont(Turbine.UI.Lotro.Font.Verdana16);
		self.CPText:SetText("");
		self.CPText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
		self.CPText:SetMouseVisible(false);
		self.CPIcon=Turbine.UI.Control();
		self.CPIcon:SetParent(self.WalletPanel);
		self.CPIcon:SetSize(27,21);
		self.CPIcon:SetPosition(146,0);
		self.CPIcon:SetBackground(0x41007e7d);
		self.CPIcon:SetMouseVisible(false);
	else
		self.Value=Turbine.UI.Label();
		self.Value:SetParent(self.WalletPanel);
		self.Value:SetSize(146,20);
		self.Value:SetPosition(0,1);
		self.Value:SetFont(Turbine.UI.Lotro.Font.Verdana16);
		self.Value:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
		self.Value:SetMouseVisible(false);
		self.Icon=Turbine.UI.Control();
		self.Icon:SetParent(self.WalletPanel);
		self.Icon:SetPosition(149,0);
		self.Icon:SetPosition(149,(20-Currency[self.CurrencyIndex][5])/2);
		self.Icon:SetSize(Currency[self.CurrencyIndex][4],Currency[self.CurrencyIndex][5]);
		self.Icon:SetBackground(Currency[self.CurrencyIndex][3]);
		self.Icon:SetMouseVisible(false);
	end

	self:SetMouseVisible(false);

	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("Wallet",self.shellCmd);

	self.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "wallet") then
			local pos1=string.find(args," ",0,true);
			local command;
			if pos1~=nil and pos1>0 then
				command=string.lower(string.sub(args,1,pos1-1));
			else
				command=string.lower(args);
			end
			if (command=="show") then
				local tmpIndex;
				for tmpIndex=1,#walletWindow do
					walletWindow[tmpIndex]:SetVisible(true);
				end
			elseif (command=="hide") then
				local tmpIndex;
				for tmpIndex=1,#walletWindow do
					walletWindow[tmpIndex]:SetVisible(false);
				end
			elseif (command=="toggle") then
				local tmpIndex;
				for tmpIndex=1,#walletWindow do
					walletWindow[tmpIndex]:SetVisible(not walletWindow[tmpIndex]:IsVisible());
				end
			elseif (command=="setup") then
				walletOptions:SetVisible(true);
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /Wallet show");
				Turbine.Shell.WriteLine("       Displays the Wallet.");
				Turbine.Shell.WriteLine("  /Wallet hide");
				Turbine.Shell.WriteLine("       Hides the Wallet.");
				Turbine.Shell.WriteLine("  /Wallet toggle");
				Turbine.Shell.WriteLine("       Toggles the Wallet (shows it if it was hidden, hides it if it was showing).");
				Turbine.Shell.WriteLine("  /Wallet setup");
				Turbine.Shell.WriteLine("       Shows the Wallet Setup Window.");
			end
		end
	end

	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /Wallet show\n       Displays the Wallet.\n  /Wallet hide\n       Hides the Wallet.\n  /Wallet toggle\n       Toggles the Wallet (shows it if it was hidden, hides it if it was showing).\n /wallet setup\n       Displays the Wallet Setup Window.");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("Wallet - usage: /Wallet show | hide | toggle | setup");
	end

	self.loaded=false;
	self:SetWantsUpdates(true);
	self:SetWantsKeyEvents(true);
	self.Changed=true;

	if Currency[self.CurrencyIndex][1]=="Gold" then
		-- these handlers are only valid for the "Gold" window
		self.Update=function()
			if self.Changed then
				local money=0;
				if localPlayer:GetAttributes().GetMoney~=nil then
					money=self.playerAttributes:GetMoney();
				end
				local cp=math.fmod(money,100);
				money=(money-cp)/100;
				local sp=math.fmod(money,1000);
				local gp=(money-sp)/1000;
				self.GPText:SetText(gp);
				-- allow for autoexpansion
				local maxWidth=Turbine.UI.Display:GetWidth()
				while self.GPText.HScrollBar:IsVisible() and self.GPText:GetWidth()<maxWidth do
					self.GPText:SetWidth(self.GPText:GetWidth()+10)
				end
				local offset=self.GPText:GetWidth()-30;
				if offset>0 then
					self:SetWidth(173+offset);
					self.MovePanel:SetWidth(173+offset)
					self.MoveText:SetWidth(156+offset);
					self.Move2:SetWidth(156+offset);
					self.WalletPanel:SetWidth(173+offset)
					self:SetLeft(walletLeft[Currency[self.CurrencyIndex]]-offset);
					self.GPIcon:SetLeft(offset+30);
					self.SPText:SetLeft(offset+62);
					self.SPIcon:SetLeft(offset+92);
					self.CPText:SetLeft(offset+124);
					self.CPIcon:SetLeft(offset+146);
				end
				self.SPText:SetText(sp);
				self.CPText:SetText(cp);

				for tmpWindow=1,#walletWindow do
					local qty=0;
					if Currency[walletWindow[tmpWindow].CurrencyIndex][1]=="Gold" then
						if localPlayer:GetAttributes().GetMoney~=nil then
							qty=self.playerAttributes:GetMoney();
						end
					elseif Currency[walletWindow[tmpWindow].CurrencyIndex][1]=="DP" then
						if localPlayer:GetAttributes().GetDestinyPoints~=nil then
							qty=self.playerAttributes:GetDestinyPoints();
							walletWindow[tmpWindow].Value:SetText(self.playerAttributes:GetDestinyPoints());
						end
					else
						if walletWindow[tmpWindow].Currency~=nil then
							qty=walletWindow[tmpWindow].Currency:GetQuantity();
							walletWindow[tmpWindow].Value:SetText(walletWindow[tmpWindow].Currency:GetQuantity());
						else
							walletWindow[tmpWindow].Value:SetText("0");
						end
					end
					local oldDisplayed=walletWindow[tmpWindow].Displayed;
					if walletSettings[tmpWindow][2] then
						if walletSettings[tmpWindow][3] then
							walletWindow[tmpWindow].Displayed=(qty~=0);
							if walletWindow[tmpWindow].Displayed then
								walletWindow[tmpWindow].WalletPanel:SetHeight(20);
							else
								walletWindow[tmpWindow].WalletPanel:SetHeight(0);
							end
						else
							walletWindow[tmpWindow].Displayed=true;
						end
					else
						walletWindow[tmpWindow].Displayed=false;
					end
					if oldDisplayed~=walletWindow[tmpWindow].Displayed then
						-- now handle the next window if it's docked (which will cascade)
						local newLeft,newTop=walletWindow[tmpWindow]:GetPosition()
						local displayIndex=walletSettings[tmpWindow][4];
						local tmpWindow2;
						for tmpWindow2=1,#walletWindow do
							if walletSettings[tmpWindow2][4]==displayIndex+1 then
								if walletSettings[tmpWindow2][5] then
									walletWindow[tmpWindow2]:SetPosition(newLeft,newTop+walletWindow[tmpWindow].WalletPanel:GetHeight());
								end
								break;
							end
						end
					end
				end

				self.Changed=false;
			end
			if not self.loaded then
				self.loaded=true;
				Plugins["Wallet"].Unload = function(self,sender,args)
					UnloadWallet();
				end
				self:SetWantsUpdates(false);
			end
		end
		self.KeyDown=function(sender, args)
			if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
				walletOptions:SetVisible(false);
			end

			if ( args.Action == 268435579 ) then -- reposition UI
				walletMoveToggle=not walletMoveToggle;
				local tmpIndex;
				for tmpIndex=1,#walletWindow do
					if walletSettings[tmpIndex][4]==1 or walletSettings[tmpIndex][5]==false then
						walletWindow[tmpIndex].MovePanel:SetVisible(walletMoveToggle);
					end
				end
			end

			if ( args.Action == 268435635 ) then -- toggle HUD
				walletHUDState = not walletHUDState;
				local tmpIndex;
				if walletHUDState then
					for tmpIndex=1,#walletWindow do
						walletWindow[tmpIndex]:SetVisible(walletState[tmpIndex]);
					end
				else
					walletOptions:SetVisible(false);
					for tmpIndex=1,#walletWindow do
						walletState[tmpIndex]=walletWindow[tmpIndex]:IsVisible();
						walletWindow[tmpIndex]:SetVisible(false);
					end
				end
			end
		end
	end
	self.playerAttributes=localPlayer:GetAttributes();

	if Currency[self.CurrencyIndex][1]=="Gold" then
		self.moneyChangedHandler = AddCallback(self.playerAttributes, "MoneyChanged", function(sender, args) self.Changed=true; self:SetWantsUpdates(true); end)
		self.dpChangedHandler = AddCallback(self.playerAttributes, "DestinyPointsChanged", function(sender, args) self.Changed=true; self:SetWantsUpdates(true); end)
	elseif self.Currency~=nil then
		self.currencyChangedHandler=AddCallback(self.Currency,"QuantityChanged",function(sender, args) walletWindow[goldWalletIndex].Changed=true; walletWindow[goldWalletIndex]:SetWantsUpdates(true); end)
	end
end
function UnloadWallet()
	PatchDataSave( Turbine.DataScope.Account, "Wallet_Currency", Currency);

	-- remove the shell command
	Turbine.Shell.RemoveCommand(walletWindow[goldWalletIndex].shellCmd)

	-- release the event handlers
	local tmpIndex;
	for tmpIndex=1,#walletWindow do
		if Currency[walletWindow[tmpIndex].CurrencyIndex][1]=="Gold" then
			RemoveCallback(walletWindow[tmpIndex].playerAttributes, "MoneyChanged", walletWindow[tmpIndex].moneyChangedHandler);
			RemoveCallback(walletWindow[tmpIndex].playerAttributes, "DestinyPointsChanged", walletWindow[tmpIndex].dpChangedHandler);
		elseif walletWindow[tmpIndex].Currency~=nil then
			RemoveCallback(walletWindow[tmpIndex].Currency, "QuantityChanged", walletWindow[tmpIndex].currencyChangedHandler);
		end
	end

--	RemoveCallback(playerWallet, "ItemAdded", walletItemAdded);
	Turbine.Shell.WriteLine("Wallet "..Plugins["Wallet"]:GetVersion().." by Garan unloaded");

	local Settings={};
	local tmpIndex;
	for tmpIndex=1,#walletSettings do
		Settings[#Settings+1]={Currency[walletSettings[tmpIndex][1]][1],{walletSettings[tmpIndex][1], walletSettings[tmpIndex][2], walletSettings[tmpIndex][3], walletSettings[tmpIndex][4], walletSettings[tmpIndex][5], walletWindow[tmpIndex]:GetLeft(), walletWindow[tmpIndex]:GetTop()}}
	end
	Settings[#Settings+1]={"Opacity",walletOpacity};

	PatchDataSave( Turbine.DataScope.Character, "Wallet_Settings", Settings );
end

walletWindow={};

local tmpIndex, tmpCurr;
for tmpIndex=1,#walletSettings do
	walletWindow[#walletWindow+1]=Wallet(tmpIndex);
end

for tmpCurr=1,#walletWindow do
	local curWind=walletWindow[tmpCurr];
	curWind.PositionChanged=function(sender,args)
		local newLeft,newTop=curWind:GetPosition()
		local displayIndex=walletSettings[curWind.Index][4];
		local tmpWindow;
		for tmpWindow=1,#walletWindow do
			if tmpWindow~=curWind.Index and walletSettings[tmpWindow][4]==displayIndex+1 then
				if walletSettings[tmpWindow][5] then
					walletWindow[tmpWindow]:SetPosition(newLeft,newTop+curWind.WalletPanel:GetHeight());
				end
				break;
			end
		end
	end
end
-- now go through the list in display order, positioning all non-docked windows (the first window is considered non-docked)
for tmpIndex=1,#walletSettings do
	if walletSettings[tmpIndex][4]==1 or walletSettings[tmpIndex][5]~=true then
		walletWindow[tmpIndex]:SetPosition(walletSettings[tmpIndex][6],walletSettings[tmpIndex][7]);
	end
end

walletOptions=Turbine.UI.Lotro.Window();
walletOptions:SetSize(566,450);
walletOptions:SetPosition((Turbine.UI.Display:GetWidth()-walletOptions:GetWidth())/2, (Turbine.UI.Display:GetHeight()-walletOptions:GetHeight())/2);
walletOptions:SetZOrder(1);
walletOptions:SetText("Wallet Setup");
walletOptions.OpacityCaption=Turbine.UI.Label();
walletOptions.OpacityCaption:SetParent(walletOptions);
walletOptions.OpacityCaption:SetSize(90,18);
walletOptions.OpacityCaption:SetPosition(10,45);
walletOptions.OpacityCaption:SetFont(Turbine.UI.Lotro.Font.Verdana18);
walletOptions.OpacityCaption:SetText("Opacity:");
walletOptions.OpacityScrollBar=Turbine.UI.Lotro.ScrollBar();
walletOptions.OpacityScrollBar:SetOrientation(Turbine.UI.Orientation.Horizontal);
walletOptions.OpacityScrollBar:SetParent(walletOptions);
walletOptions.OpacityScrollBar:SetSize(walletOptions:GetWidth()-walletOptions.OpacityCaption:GetLeft()-walletOptions.OpacityCaption:GetWidth()-15,10);
walletOptions.OpacityScrollBar:SetPosition(100,walletOptions.OpacityCaption:GetTop()+5);
walletOptions.OpacityScrollBar:SetMinimum(0);
walletOptions.OpacityScrollBar:SetMaximum(100);
walletOptions.OpacityScrollBar:SetValue(walletOpacity*100);
walletOptions.OpacityScrollBar.ValueChanged=function()
	walletOpacity=walletOptions.OpacityScrollBar:GetValue()/100;
	local tmpIndex;
	for tmpIndex=1,#walletWindow do
		walletWindow[tmpIndex]:SetOpacity(walletOpacity);
	end
end
walletOptions.CP=Turbine.UI.Control();
walletOptions.CP:SetParent(walletOptions);
walletOptions.CP:SetPosition(10,walletOptions.OpacityCaption:GetTop()+walletOptions.OpacityCaption:GetHeight()+5);
walletOptions.CP:SetSize(walletOptions:GetWidth()-20,walletOptions:GetHeight()-walletOptions.CP:GetTop()-45);
walletOptions.CP:SetBackColor(Turbine.UI.Color(0,0,.3));
walletOptions.CP.NameCaption=Turbine.UI.Label();
walletOptions.CP.NameCaption:SetParent(walletOptions.CP);
walletOptions.CP.NameCaption:SetSize(250,20);
walletOptions.CP.NameCaption:SetPosition(1,1);
walletOptions.CP.NameCaption:SetFont(Turbine.UI.Lotro.Font.Verdana18);
walletOptions.CP.NameCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
walletOptions.CP.NameCaption:SetText("Currency");
walletOptions.CP.NameCaption:SetBackColor(Turbine.UI.Color(0,0,0));
walletOptions.CP.ShowCaption=Turbine.UI.Label();
walletOptions.CP.ShowCaption:SetParent(walletOptions.CP);
walletOptions.CP.ShowCaption:SetSize(70,20);
walletOptions.CP.ShowCaption:SetPosition(walletOptions.CP.NameCaption:GetLeft()+walletOptions.CP.NameCaption:GetWidth()+1,1);
walletOptions.CP.ShowCaption:SetFont(Turbine.UI.Lotro.Font.Verdana18);
walletOptions.CP.ShowCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
walletOptions.CP.ShowCaption:SetText("Display");
walletOptions.CP.ShowCaption:SetBackColor(Turbine.UI.Color(0,0,0));
walletOptions.CP.Hide0Caption=Turbine.UI.Label();
walletOptions.CP.Hide0Caption:SetParent(walletOptions.CP);
walletOptions.CP.Hide0Caption:SetSize(70,20);
walletOptions.CP.Hide0Caption:SetPosition(walletOptions.CP.ShowCaption:GetLeft()+walletOptions.CP.ShowCaption:GetWidth()+1,1);
walletOptions.CP.Hide0Caption:SetFont(Turbine.UI.Lotro.Font.Verdana18);
walletOptions.CP.Hide0Caption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
walletOptions.CP.Hide0Caption:SetText("Hide if 0");
walletOptions.CP.Hide0Caption:SetBackColor(Turbine.UI.Color(0,0,0));
walletOptions.CP.DockCaption=Turbine.UI.Label();
walletOptions.CP.DockCaption:SetParent(walletOptions.CP);
walletOptions.CP.DockCaption:SetSize(70,20);
walletOptions.CP.DockCaption:SetPosition(walletOptions.CP.Hide0Caption:GetLeft()+walletOptions.CP.Hide0Caption:GetWidth()+1,1);
walletOptions.CP.DockCaption:SetFont(Turbine.UI.Lotro.Font.Verdana18);
walletOptions.CP.DockCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
walletOptions.CP.DockCaption:SetText("Dock");
walletOptions.CP.DockCaption:SetBackColor(Turbine.UI.Color(0,0,0));
walletOptions.CP.OrderCaption=Turbine.UI.Label();
walletOptions.CP.OrderCaption:SetParent(walletOptions.CP);
walletOptions.CP.OrderCaption:SetSize(70,20);
walletOptions.CP.OrderCaption:SetPosition(walletOptions.CP.DockCaption:GetLeft()+walletOptions.CP.DockCaption:GetWidth()+1,1);
walletOptions.CP.OrderCaption:SetFont(Turbine.UI.Lotro.Font.Verdana18);
walletOptions.CP.OrderCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
walletOptions.CP.OrderCaption:SetText("Order");
walletOptions.CP.OrderCaption:SetBackColor(Turbine.UI.Color(0,0,0));
walletOptions.CP.CurrencyList=Turbine.UI.ListBox();
walletOptions.CP.CurrencyList:SetParent(walletOptions.CP);
walletOptions.CP.CurrencyList:SetPosition(1,22);
walletOptions.CP.CurrencyList:SetSize(walletOptions.CP:GetWidth()-12,walletOptions.CP:GetHeight()-23);
walletOptions.CP.CurrencyList:SetBackColor(Turbine.UI.Color(0,0,0));

walletOptions.CP.VScroll=Turbine.UI.Lotro.ScrollBar();
walletOptions.CP.VScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
walletOptions.CP.VScroll:SetParent(walletOptions.CP);
walletOptions.CP.VScroll:SetSize(10,walletOptions.CP.CurrencyList:GetHeight());
walletOptions.CP.VScroll:SetPosition(walletOptions.CP.CurrencyList:GetLeft()+walletOptions.CP.CurrencyList:GetWidth(),walletOptions.CP.CurrencyList:GetTop());
walletOptions.CP.CurrencyList:SetVerticalScrollBar(walletOptions.CP.VScroll);
walletOptions.UpEnabledImage="GaranStuff/Wallet/Resources/UpArrow.tga";
walletOptions.UpDisabledImage="GaranStuff/Wallet/Resources/UpArrow_grey.tga";
walletOptions.DownEnabledImage="GaranStuff/Wallet/Resources/DownArrow.tga";
walletOptions.DownDisabledImage="GaranStuff/Wallet/Resources/DownArrow_grey.tga";

walletOptions.Update=function()
	local size=playerWallet:GetSize();

	if size>walletSize then
		walletSize=size;
		local tmpCurrency, tmpWallet;
		for tmpCurrency=1,size do
			local found=false;
			for tmpWallet=1,#walletSettings do
				if Currency[walletSettings[tmpWallet][1]][1]==playerWallet:GetItem(tmpCurrency):GetName() then
					found=true;
					if walletWindow[tmpWallet].Currency==nil then
						walletWindow[tmpWallet].Currency=playerWallet:GetItem(tmpCurrency);
						walletWindow[tmpWallet].currencyChangedHandler=AddCallback(walletWindow[tmpWallet].Currency,"QuantityChanged",function(sender, args) walletWindow[goldWalletIndex].Changed=true; walletWindow[goldWalletIndex]:SetWantsUpdates(true); end)
						if walletSettings[tmpWallet][2] then
							if walletSettings[tmpWallet][3] then
								local curQty=0;
								local tmpIndex;
								if Currency[walletWindow[tmpWallet].CurrencyIndex][1]=="Gold" then
									if localPlayer:GetAttributes().GetMoney~=nil then
										curQty=localPlayer:GetAttributes():GetMoney();
									end
								elseif Currency[walletWindow[tmpWallet].CurrencyIndex][1]=="DP" then
									if localPlayer:GetAttributes().GetDestinyPoints~=nil then
										curQty=localPlayer:GetAttributes():GetDestinyPoints();
									end
								else
									if walletWindow[tmpWallet].Currency~=nil then
										curQty=walletWindow[tmpWallet].Currency:GetQuantity();
									end
								end
								walletWindow[tmpWallet].Displayed=(curQty~=0);
							end
						else
							walletWindow[tmpWallet].Displayed=false;
						end
						if walletWindow[tmpWallet].Displayed then
							walletWindow[tmpWallet].WalletPanel:SetHeight(20);
						else
							walletWindow[tmpWallet].WalletPanel:SetHeight(0);
						end
						-- now handle the next window if it's docked (which will cascade)
						local newLeft,newTop=walletWindow[tmpWallet]:GetPosition()
						local displayIndex=walletSettings[tmpWallet][4];
						local tmpWindow;
						for tmpWindow=1,#walletWindow do
							if walletSettings[tmpWindow][4]==displayIndex+1 then
								if walletSettings[tmpWindow][5] then
									walletWindow[tmpWindow]:SetPosition(newLeft,newTop+walletWindow[tmpWallet].WalletPanel:GetHeight());
								end
								break;
							end
						end
					end
					break;
				end
			end
			if not found then
				CurrencyAdded(tmpCurrency);
			end
		end
		-- now trigger an update
		walletWindow[goldWalletIndex].Changed=true;
		walletWindow[goldWalletIndex]:SetWantsUpdates(true);
	end
end
walletOptions:SetWantsUpdates(true);

function CreateOptionsRow(displayIndex,tmpIndex)
	local tmpRow=Turbine.UI.Control()
	tmpRow.CurrencyIndex=walletSettings[tmpIndex][1];
	tmpRow.DisplayIndex=displayIndex;
	tmpRow.WindowIndex=tmpIndex;
	tmpRow:SetParent(walletOptions.CP.CurrencyList);
	tmpRow:SetSize(walletOptions.CP.CurrencyList:GetWidth(),20);
	if tmpIndex/2==math.floor(tmpIndex/2) then
		tmpRow:SetBackColor(Turbine.UI.Color(.05,.05,.05));
	else
		tmpRow:SetBackColor(Turbine.UI.Color(.15,.15,.15));
	end
	tmpRow.Name=Turbine.UI.Label();
	tmpRow.Name:SetParent(tmpRow);
	tmpRow.Name:SetSize(walletOptions.CP.NameCaption:GetWidth(),20);
	tmpRow.Name:SetPosition(walletOptions.CP.NameCaption:GetLeft()-1,0);
	tmpRow.Name:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	tmpRow.Name:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	tmpRow.Name:SetText(Currency[tmpRow.CurrencyIndex][1])
		tmpRow.Icon=Turbine.UI.Control();
	tmpRow.Icon:SetParent(tmpRow);
	tmpRow.Icon:SetSize(16,16);
	tmpRow.Icon:SetPosition(tmpRow.Name:GetLeft()+tmpRow.Name:GetWidth(),4);
	tmpRow.Icon:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	tmpRow.Icon:SetLeft(tmpRow.Name:GetLeft()+tmpRow.Name:GetWidth());
	tmpRow.Icon:SetSize(Currency[tmpRow.CurrencyIndex][4],Currency[tmpRow.CurrencyIndex][5]);
	tmpRow.Icon:SetBackground(Currency[tmpRow.CurrencyIndex][3]);
		tmpRow.Show=Turbine.UI.Lotro.CheckBox();
	tmpRow.Show:SetParent(tmpRow);
	tmpRow.Show:SetSize(20,20);
	tmpRow.Show:SetPosition(walletOptions.CP.ShowCaption:GetLeft()+walletOptions.CP.ShowCaption:GetWidth()/2-10,0);
	tmpRow.Show:SetText("");
	tmpRow.Show:SetChecked(walletSettings[tmpIndex][2]);
	tmpRow.Show.CheckedChanged=function()
		walletSettings[tmpRow.WindowIndex][2]=tmpRow.Show:IsChecked();
		local displayed=true;
		if walletSettings[tmpRow.WindowIndex][2] then
			if walletSettings[tmpRow.WindowIndex][3] then
				if Currency[tmpRow.CurrencyIndex][1]=="Gold" then
					if localPlayer:GetAttributes().GetMoney==nil or localPlayer:GetAttributes():GetMoney()==0 then
						displayed=false;
					end
				elseif Currency[tmpRow.CurrencyIndex][1]=="DP" then
					if localPlayer:GetAttributes().GetDestinyPoints~=nil or localPlayer:GetAttributes():GetDestinyPoints()==0 then
						displayed=false;
					end
				else
					if tonumber(walletWindow[tmpRow.WindowIndex].Value:GetText())==0 then
						displayed=false;
					end
				end
			end
		else
			displayed=false;
		end
		walletWindow[tmpRow.WindowIndex].Displayed=displayed;
		if displayed then
			walletWindow[tmpRow.WindowIndex].WalletPanel:SetHeight(20);
		else
			walletWindow[tmpRow.WindowIndex].WalletPanel:SetHeight(0);
		end
		-- now, if the following window is docked, set it's position based on this position + height
		local tmpWallet
		for tmpWallet=1,#walletWindow do
			if walletSettings[tmpWallet][4]==walletSettings[tmpRow.WindowIndex][4]+1 and walletSettings[tmpWallet][5] then
				walletWindow[tmpWallet]:SetPosition(walletWindow[tmpRow.WindowIndex]:GetLeft(),walletWindow[tmpRow.WindowIndex]:GetTop()+walletWindow[tmpRow.WindowIndex].WalletPanel:GetHeight());
				break;
			end
		end
	end
	tmpRow.Hide0=Turbine.UI.Lotro.CheckBox();
	tmpRow.Hide0:SetParent(tmpRow);
	tmpRow.Hide0:SetSize(20,20);
	tmpRow.Hide0:SetPosition(walletOptions.CP.Hide0Caption:GetLeft()+walletOptions.CP.Hide0Caption:GetWidth()/2-10,0);
	tmpRow.Hide0:SetText("");
	tmpRow.Hide0:SetChecked(walletSettings[tmpIndex][3]);
	tmpRow.Hide0.CheckedChanged=function()
		walletSettings[tmpRow.WindowIndex][3]=tmpRow.Hide0:IsChecked();
		local displayed=true;
		if walletSettings[tmpRow.WindowIndex][2] then
			if walletSettings[tmpRow.WindowIndex][3] then
				if Currency[tmpRow.CurrencyIndex][1]=="Gold" then
					if localPlayer:GetAttributes().GetMoney==nil or localPlayer:GetAttributes():GetMoney()==0 then
						displayed=false;
					end
				elseif Currency[tmpRow.CurrencyIndex][1]=="DP" then
					if localPlayer:GetAttributes().GetDestinyPoints~=nil or localPlayer:GetAttributes():GetDestinyPoints()==0 then
						displayed=false;
					end
				else
					if tonumber(walletWindow[tmpRow.WindowIndex].Value:GetText())==0 then
						displayed=false;
					end
				end
			end
		else
			displayed=false;
		end
		walletWindow[tmpRow.WindowIndex].Displayed=displayed;
		if displayed then
			walletWindow[tmpRow.WindowIndex].WalletPanel:SetHeight(20);
		else
			walletWindow[tmpRow.WindowIndex].WalletPanel:SetHeight(0);
		end
		-- now, if the following window is docked, set it's position based on this position + height
		local tmpWallet
		for tmpWallet=1,#walletWindow do
			if walletSettings[tmpWallet][4]==walletSettings[tmpRow.WindowIndex][4]+1 and walletSettings[tmpWallet][5] then
				walletWindow[tmpWallet]:SetPosition(walletWindow[tmpRow.WindowIndex]:GetLeft(),walletWindow[tmpRow.WindowIndex]:GetTop()+walletWindow[tmpRow.WindowIndex].WalletPanel:GetHeight());
				break;
			end
		end
	end
	tmpRow.Dock=Turbine.UI.Lotro.CheckBox();
	tmpRow.Dock:SetParent(tmpRow);
	tmpRow.Dock:SetSize(20,20);
	tmpRow.Dock:SetPosition(walletOptions.CP.DockCaption:GetLeft()+walletOptions.CP.DockCaption:GetWidth()/2-10,0);
	tmpRow.Dock:SetText("");
	tmpRow.Dock:SetChecked(walletSettings[tmpIndex][5]);
	tmpRow.Dock.CheckedChanged=function()
		walletSettings[tmpRow.WindowIndex][5]=tmpRow.Dock:IsChecked();
		if walletSettings[tmpRow.WindowIndex][5] then
			if walletSettings[tmpRow.WindowIndex][4]>1 then
				local tmpWallet
				for tmpWallet=1,#walletWindow do
					if walletSettings[tmpWallet][4]==walletSettings[tmpRow.WindowIndex][4]-1 then
						walletWindow[tmpRow.WindowIndex]:SetPosition(walletWindow[tmpWallet]:GetLeft(),walletWindow[tmpWallet]:GetTop()+walletWindow[tmpWallet].WalletPanel:GetHeight());
						break;
					end
				end
			end
		end
		if walletSettings[tmpRow.WindowIndex][4]==1 or walletSettings[tmpRow.WindowIndex][5]==false then
			walletWindow[tmpRow.WindowIndex].MovePanel:SetVisible(walletMoveToggle);
		else
			walletWindow[tmpRow.WindowIndex].MovePanel:SetVisible(false);
		end
	end
	tmpRow.MoveUp=Turbine.UI.Control();
	tmpRow.MoveUp:SetParent(tmpRow);
	tmpRow.MoveUp:SetSize(18,18);
	tmpRow.MoveUp:SetPosition(walletOptions.CP.OrderCaption:GetLeft(),1);
	if walletSettings[tmpIndex][4]<2 then
		tmpRow.MoveUp:SetBackground(walletOptions.UpDisabledImage);
	else
		tmpRow.MoveUp:SetBackground(walletOptions.UpEnabledImage);
	end
	tmpRow.MoveUp:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	tmpRow.MoveUp.MouseClick=function()
		if walletSettings[tmpRow.WindowIndex][4]>1 then
			local priorRow, nextRow, displayIndex=nil,nil,tmpRow.DisplayIndex;
			if tmpRow.DisplayIndex>1 then
				priorRow=walletOptions.CP.CurrencyList:GetItem(displayIndex-1);
			end
			if tmpRow.DisplayIndex<walletOptions.CP.CurrencyList:GetItemCount() then
				nextRow=walletOptions.CP.CurrencyList:GetItem(displayIndex-1);
			end
				walletSettings[priorRow.WindowIndex][4]=displayIndex;
			walletSettings[tmpRow.WindowIndex][4]=displayIndex-1;
			priorRow.Order:SetText(displayIndex);
			tmpRow.Order:SetText(displayIndex-1);
			-- now to actually flip the rows
			walletOptions.CP.CurrencyList:RemoveItemAt(displayIndex); --remove this row
			walletOptions.CP.CurrencyList:InsertItem(displayIndex-1,tmpRow); -- reinsert
			tmpRow.DisplayIndex=displayIndex-1;
			priorRow.DisplayIndex=displayIndex;
			-- fix backgrounds
			if displayIndex/2==math.floor(displayIndex/2) then
				tmpRow:SetBackColor(Turbine.UI.Color(.15,.15,.15));
				priorRow:SetBackColor(Turbine.UI.Color(.05,.05,.05));
			else
				tmpRow:SetBackColor(Turbine.UI.Color(.05,.05,.05));
				priorRow:SetBackColor(Turbine.UI.Color(.15,.15,.15));
			end
			if displayIndex==2 then
				tmpRow.MoveUp:SetBackground(walletOptions.UpDisabledImage);
				priorRow.MoveUp:SetBackground(walletOptions.UpEnabledImage);
			end
			-- now we have to reposition the actual windows
			-- this is the lazy way
			local tmpWind
			for tmpWind=1,#walletSettings do
				walletWindow[tmpWind].PositionChanged();
			end
		end
	end
	tmpRow.Order=Turbine.UI.Label();
	tmpRow.Order:SetParent(tmpRow)
	tmpRow.Order:SetSize(walletOptions.CP.OrderCaption:GetWidth()-36,18);
	tmpRow.Order:SetPosition(walletOptions.CP.OrderCaption:GetLeft()+18,1);
	tmpRow.Order:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
	tmpRow.Order:SetText(walletSettings[tmpIndex][4]);
		tmpRow.MoveDown=Turbine.UI.Control();
	tmpRow.MoveDown:SetParent(tmpRow);
	tmpRow.MoveDown:SetSize(18,18);
	tmpRow.MoveDown:SetPosition(walletOptions.CP.OrderCaption:GetLeft()+walletOptions.CP.OrderCaption:GetWidth()-18,1);
	if walletSettings[tmpRow.CurrencyIndex][4]>=#walletSettings then
		tmpRow.MoveDown:SetBackground(walletOptions.DownDisabledImage);
	else
		tmpRow.MoveDown:SetBackground(walletOptions.DownEnabledImage);
	end
	tmpRow.MoveDown:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	tmpRow.MoveDown.MouseClick=function()
		if walletSettings[tmpRow.WindowIndex][4]<#walletSettings then
			local priorRow, nextRow, displayIndex=nil,nil,tmpRow.DisplayIndex;
			if tmpRow.DisplayIndex>1 then
				priorRow=walletOptions.CP.CurrencyList:GetItem(displayIndex-1);
			end
			if tmpRow.DisplayIndex<walletOptions.CP.CurrencyList:GetItemCount() then
				nextRow=walletOptions.CP.CurrencyList:GetItem(displayIndex+1);
			end
			walletSettings[nextRow.WindowIndex][4]=displayIndex;
			walletSettings[tmpRow.WindowIndex][4]=displayIndex+1;
			nextRow.Order:SetText(displayIndex);
			tmpRow.Order:SetText(displayIndex+1);
			-- now to actually flip the rows
			walletOptions.CP.CurrencyList:RemoveItemAt(displayIndex); --remove this row
			walletOptions.CP.CurrencyList:InsertItem(displayIndex+1,tmpRow); -- reinsert
			tmpRow.DisplayIndex=displayIndex+1;
			nextRow.DisplayIndex=displayIndex;
			-- fix backgrounds
			if displayIndex/2==math.floor(displayIndex/2) then
				tmpRow:SetBackColor(Turbine.UI.Color(.15,.15,.15));
				nextRow:SetBackColor(Turbine.UI.Color(.05,.05,.05));
			else
				tmpRow:SetBackColor(Turbine.UI.Color(.05,.05,.05));
				nextRow:SetBackColor(Turbine.UI.Color(.15,.15,.15));
			end
			if displayIndex==#walletSettings-1 then
				tmpRow.MoveDown:SetBackground(walletOptions.DownDisabledImage);
				nextRow.MoveDown:SetBackground(walletOptions.DownEnabledImage);
			end
			-- now we have to reposition the actual windows
			-- this is the lazy way
			local tmpWind
			for tmpWind=1,#walletSettings do
				walletWindow[tmpWind].PositionChanged();
			end
		end
	end
	return (tmpRow)
end
for displayIndex=1,#walletSettings do
	for tmpIndex=1,#walletSettings do
		if walletSettings[tmpIndex][4]==displayIndex then
			walletOptions.CP.CurrencyList:AddItem(CreateOptionsRow(displayIndex,tmpIndex));
		end
	end
end
-- handle currencies that are added while playing
function CurrencyAdded(index)
	local newCurrency=playerWallet:GetItem(index);
	if AddCurrency(newCurrency) then
		local newIndex=#walletSettings+1
		local left,top=0,#Currency*20;
		local priorWindow;
		for priorWindow=1,#walletSettings do
			if walletSettings[priorWindow][4]==newIndex-1 then
				left=walletWindow[priorWindow]:GetLeft();
				top=walletWindow[priorWindow]:GetTop()+walletWindow[priorWindow].WalletPanel:GetHeight();
			end
		end
		walletSettings[newIndex]={#Currency, newCurrency:IsAccountItem(), true, newIndex, true, left, top};
		walletWindow[newIndex]=Wallet(newIndex);
		-- now add an entry in Options window
		-- enable the "move down" button of the old last row
		walletOptions.CP.CurrencyList:GetItem(walletOptions.CP.CurrencyList:GetItemCount()).MoveDown:SetBackground(walletOptions.DownEnabledImage);
		walletOptions.CP.CurrencyList:AddItem(CreateOptionsRow(newIndex,newIndex));
	end
end
--walletItemAdded=AddCallback(playerWallet,"ItemAdded",function(sender,args) CurrencyAdded(args.Index) end)
